#!/usr/bin/perl -w
#
# ===========================================================================
#
# Class for Benchmark Table Management
#
# Date: 14 Aug 2003
# Author: Jerome Delamarche (c) Copyright INODES.  2003  All rights reserved.
# Version: 2.0
#
# ---------------------------------------------------------------------------
#
# TODO: an new function that contains the last DB error ?
#		
# ===========================================================================
#
use DBI;
use Socket;
use strict;
require "util.inc.pl";

# ===================================================
# This is the main class that extracts data from 
# the benchmark database
# ===================================================
package CDBBenchmark;

# common queries:
my %m_avgQuery = (
		"apache" => "SELECT AVG(BEN_BUSY) AS VALUE FROM T_BENCHMARK_LOG",
		"cpu"	 => "SELECT AVG(BEN_AVGLOAD) AS VALUE FROM T_BENCHMARK_LOG",
		"mem"	 => "SELECT AVG(BEN_MEMUSE) AS VALUE FROM T_BENCHMARK_LOG",
		"swap"	 => "SELECT AVG(BEN_SWAPUSE) AS VALUE FROM T_BENCHMARK_LOG",
		"time"   => "SELECT AVG(BEN_SEC * 1000 + BEN_USEC / 1000) AS VALUE FROM T_BENCHMARK_LOG",
		"alert"  => "SELECT COUNT(*) AS VALUE FROM T_BENCHMARK_ALERT",
		);

my %m_maxQuery = (
		"apache" => "SELECT MAX(BEN_BUSY) AS VALUE FROM T_BENCHMARK_LOG",
		"cpu"	 => "SELECT MAX(BEN_AVGLOAD) AS VALUE FROM T_BENCHMARK_LOG",
		"mem"	 => "SELECT MAX(BEN_MEMUSE) AS VALUE FROM T_BENCHMARK_LOG",
		"swap"	 => "SELECT MAX(BEN_SWAPUSE) AS VALUE FROM T_BENCHMARK_LOG",
		"time"   => "SELECT MAX(BEN_SEC * 1000 + BEN_USEC / 1000) AS VALUE FROM T_BENCHMARK_LOG",
		"alert"  => "SELECT COUNT(*) AS VALUE FROM T_BENCHMARK_ALERT",
		);

my %m_minQuery = (
		"apache" => "SELECT MIN(BEN_BUSY) AS VALUE FROM T_BENCHMARK_LOG",
		"cpu"	 => "SELECT MIN(BEN_AVGLOAD) AS VALUE FROM T_BENCHMARK_LOG",
		"mem"	 => "SELECT MIN(BEN_MEMUSE) AS VALUE FROM T_BENCHMARK_LOG",
		"swap"	 => "SELECT MIN(BEN_SWAPUSE) AS VALUE FROM T_BENCHMARK_LOG",
		"time"   => "SELECT MIN(BEN_SEC * 1000 + BEN_USEC / 1000) AS VALUE FROM T_BENCHMARK_LOG",
		"alert"  => "SELECT COUNT(*) AS VALUE FROM T_BENCHMARK_ALERT",
		);

# ===================================================
# Since SQL language are different, we must
# implement some small methods that gives the exact
# SQL syntax for the WHERE clause:
#
# XXX_GetHourPart($field)
# XXX_GetMinutePart($field)
# XXX_GetSecondPart($field)
# XXX_GetDayOfWeek($field)
# XXX_GetWeek($field)
# XXX_GetDayOfMonth($field)
# XXX_GetMonth($field)
# XXX_GetYear($field)
# XXX_DateCompare($field,$date1,$date2)
# ===================================================

# We define here the small functions that handle the Database differences:
my %m_db_features = (
		"MySQL" => {
			"GetHourPart" => sub { my ($f) = @_; return "HOUR($f)"; },
			"GetMinutePart" => sub { my ($f) = @_; return "MINUTE($f)"; },
			"GetSecondPart" => sub { my ($f) = @_; return "SECOND($f)"; },
			"GetDayOfWeek" => sub { my ($f) = @_; return "DAYOFWEEK($f)"; },
			"GetWeek" => sub { my ($f) = @_; return "WEEK($f)"; },
			"GetDayOfMonth" => sub { my ($f) = @_; return "DAYOFMONTH($f)"; },
			"GetMonth" => sub { my ($f) = @_; return "MONTH($f)"; },
			"GetYear" => sub { my ($f) = @_; return "YEAR($f)"; },
			"DateCompare" => sub { my ($f,$d1,$d2) = @_; return "$f > \"$d1\" AND $f <= \"$d2\""; },
		},
		"PostgreSQL" => {
			"GetHourPart" => sub { my ($f) = @_; return "date_part('hour',$f)"; },
			"GetMinutePart" => sub { my ($f) = @_; return "date_part('min',$f)"; },
			"GetSecondPart" => sub { my ($f) = @_; return "date_part('sec',$f)"; },
			"GetDayOfWeek" => sub { my ($f) = @_; return "date_part('dow',$f)"; },
			"GetWeek" => sub { my ($f) = @_; return "date_part('week',$f)"; },
			"GetDayOfMonth" => sub { my ($f) = @_; return "date_part('day',$f)"; },
			"GetMonth" => sub { my ($f) = @_; return "date_part('month',$f)"; },
			"GetYear" => sub { my ($f) = @_; return "date_part('year',$f)"; },
			"DateCompare" => sub { my ($f,$d1,$d2) = @_; return "$f > '$d1' AND $f <= '$d2'"; },
		},
		"Sybase" => {
			"GetHourPart" => sub { my ($f) = @_; return "datepart(hour,$f)"; },
			"GetMinutePart" => sub { my ($f) = @_; return "datepart(minute,$f)"; },
			"GetSecondPart" => sub { my ($f) = @_; return "datepart(second,$f)"; },
			"GetDayOfWeek" => sub { my ($f) = @_; return "datepart(weekday,$f)"; },
			"GetWeek" => sub { my ($f) = @_; return "datepart(week,$f)"; },
			"GetDayOfMonth" => sub { my ($f) = @_; return "datepart(day,$f)"; },
			"GetMonth" => sub { my ($f) = @_; return "datepart(month,$f)"; },
			"GetYear" => sub { my ($f) = @_; return "datepart(year,$f)"; },
			"DateCompare" => sub {
				my ($f,$d1,$d2) = @_;
				my ($y1,$m1,$h1,$i1,$s1,$y2,$m2,$h2,$i2,$s2); 
				($y1,$m1,$d1,$h1,$i1,$s1) = split(/[-: ]/,$d1);
				$m1 = int($m1);
				($y2,$m2,$d2,$h2,$i2,$s2) = split(/[-: ]/,$d2);
				$m2 = int($m2);
				return "$f > \"$m1/$d1/$y1 $h1:$i1:$s1\" AND $f <= \"$m2/$d2/$y2 $h2:$i2:$s2\""; 
			},
		},
		"Oracle" => {
			"GetHourPart" => sub { my ($f) = @_; return "TO_CHAR($f,'HH24')"; },
			"GetMinutePart" => sub { my ($f) = @_; return "TO_CHAR($f,'MI')"; },
			"GetSecondPart" => sub { my ($f) = @_; return "TO_CHAR(,$f,'SS')"; },
			"GetDayOfWeek" => sub { my ($f) = @_; return "TO_CHAR($f,'D')"; },
			"GetWeek" => sub { my ($f) = @_; return "TO_CHAR($f,'WW')"; },
			"GetDayOfMonth" => sub { my ($f) = @_; return "TO_CHAR($f,'DD')"; },
			"GetMonth" => sub { my ($f) = @_; return "TO_CHAR($f,'MM')"; },
			"GetYear" => sub { my ($f) = @_; return "TO_CHAR($f,'YYYY')"; },
			"DateCompare" => sub {
				my ($f,$d1,$d2) = @_;
				my ($y1,$m1,$h1,$i1,$s1,$y2,$m2,$h2,$i2,$s2);
				($y1,$m1,$d1,$h1,$i1,$s1) = split(/[-: ]/,$d1);
				$m1 = int($m1);
				($y2,$m2,$d2,$h2,$i2,$s2) = split(/[- :]/,$d2);
				$m2 = int($m2);
				return "$f > TO_DATE('$m1/$d1/$y1 $h1:$i1:$s1','MM/DD/YYYY HH24:MI:SS') AND $f <= TO_DATE('$m2/$d2/$y2 $h2:$i2:$s2','MM/DD/YYYY HH24:MI:SS')"; 
			},
		},
	);

# ----------
# Ctor:
# ----------
sub new
{
	my $class = shift;
	my ($dbtype,$dbname,$dbserver,$dbuser,$dbpwd) = @_;
	my $self = {};
	bless($self);

	$self->{'DBTYPE'} = $dbtype;
	$self->{'DBNAME'} = $dbname;
	$self->{'DBH'}    = undef;

	# filter values:
	$self->{'m_startdate'} = "2002-01-01 00:00:00";	# start date for stat computation
	$self->{'m_enddate'}   = "2010-12-31 23:59:59";	# end date
	$self->{'m_filterurl'} = "all";
	$self->{'m_threshold'} = 0;
	$self->{'m_servers'}   = "all";
	$self->{'m_protocols'} = "all";

	my $dsn;

	if ($dbtype eq "MySQL") { $dsn = "DBI:mysql:$dbname:$dbserver"; }
	elsif ($dbtype eq "Sybase") { $dsn = "DBI:Sybase:server=$dbserver"; }
	elsif ($dbtype eq "PostgreSQL") { $dsn = "DBI:Pg:dbname=$dbname"; }
	elsif ($dbtype eq "Oracle") { $dsn = "DBI:Oracle:"; }
	else { die "Database '$dbtype' not supported\n"; }

	$self->{'DBH'} = DBI->connect($dsn,$dbuser,$dbpwd);
	if (!defined($self->{'DBH'})) {
		die "Could not connect with DSN: $dsn, user=$dbuser, pwd=$dbpwd. Error=$DBI::errstr\n";
	}

	#$self->{'DBH'}->trace(1,"/var/log/dbi.log");

	bless($self,$class);
	return $self;
}

# ---------------------
# Return the last error:
# ---------------------
sub LastError
{
	return $DBI::errstr;
}

# -------------------------------------
# Generate a more precise error message:
# -------------------------------------
sub ErrorLog
{
	my ($self,$text) = @_;
	print STDERR "$self->{'DBNAME'}: $text\n";
	print STDERR &LastError()."\n";
}

# ------------------------------------------------
# Execute a query that sould not return a row 
# (DELETE, UPDATE, INSERT)
# ------------------------------------------------
sub QueryExecute
{
	my ($self,$query) = @_;
	my $sth = $self->{'DBH'}->prepare($query);
	if (!defined($sth)) { return undef; }

	my $res = $sth->execute();
	$sth->finish();

	return $res;
}

# ------------------------------------------------
# Execute a query that should return multiple rows:
# ------------------------------------------------
sub QueryAndFetchRows
{
	my ($self,$query) = @_;
	my $sth = $self->{'DBH'}->prepare($query);
	if (!defined($sth)) { return undef; }
	my $res = $sth->execute();
	if (!defined($res)) { return undef; }

	$res = $sth->fetchall_arrayref({});

	# PostgreSQL always returns column names in lower case !
	if ($self->{'DBTYPE'} eq "PostgreSQL") {
		my $idx;
		for ( $idx = 0 ; $idx < scalar(@{$res}) ; $idx++ ) {
			my $row = $res->[$idx];
			my $key;
			my $value;
			while (($key,$value) = each(%{$row})) {
				$row->{uc($key)} = $value;
			}
		}
	}

	$sth->finish();

	return $res;
}

# -------------------------------------------------
# Execute a query and get the 1st returned row only:
# -------------------------------------------------
sub QueryAndFetchaRow
{
	my ($self,$query) = @_;
	my $row = $self->{'DBH'}->selectrow_hashref($query);

	# PostgreSQL always returns column names in lower case !
	if ($self->{'DBTYPE'} eq "PostgreSQL") {
		my $key;
		my $value;
		while (($key,$value) = each(%{$row})) {
			$row->{uc($key)} = $value;
		}
	}

	return $row;
}

# ---------------------------------------------------------
# Used by the menu to display the list of all available URL:
# ---------------------------------------------------------
sub GetURLList
{
	my $self = shift;

	my $query = "SELECT DISTINCT URL_VALUE FROM T_BENCHMARK_LOG ORDER BY URL_VALUE ASC";
	my $result = $self->QueryAndFetchRows($query);
	if (!defined($result)) {
		$self->ErrorLog("CDBBenchmark::URLList() failed");
		return 0;
	}

	# for early debugging purpose:
	#my $rows = scalar(@{$result});
	#for ( my $i = 0 ; $i < $rows ; $i++ ) {
	#	print "$i: ".$result->[$i]{'url_value'}."\n";
	#}

	return $result;
}

# -------------------------------------------------------------
# Used by the menu to display the list of all available Servers:
# -------------------------------------------------------------
sub GetServerList
{
	my $self = shift;

	my $query = "SELECT DISTINCT BEN_SRVADDR FROM T_BENCHMARK_LOG ORDER BY BEN_SRVADDR ASC";
	my $result = $self->QueryAndFetchRows($query);
	if (!defined($result)) {
		$self->ErrorLog("CDBBenchmark::GetServerList() failed");
		return 0;
	}

	return $result;
}

# ---------------------------------------------------------------
# Used by the menu to display the list of all available Protocols:
# ---------------------------------------------------------------
sub GetProtocolList
{
	my $self = shift;

	my $query = "SELECT DISTINCT BEN_SRVPORT FROM T_BENCHMARK_LOG ORDER BY BEN_SRVPORT ASC";
	my $result = $self->QueryAndFetchRows($query);
	if (!defined($result)) {
		$self->ErrorLog("CDBBenchmark::GetProtocolList() failed");
		return 0;
	}

	return $result;
}

# =================================================
# Stored Graph Management:
# =================================================

# ----------------------------------------------
# Return the content of T_BENCHMARK_GRAPH table:
# ----------------------------------------------
sub GetGraphList
{
	my $self = shift;

	# graphs contains BLOB:
	if ($self->{'DBTYPE'} eq "Sybase") {
		$self->{'DBH'}->do('set textsize 100000');
	}
	else {
		$self->{'DBH'}->{'LongReadLen'} = 100000;
	}

	my $query = "SELECT * FROM T_BENCHMARK_GRAPH ORDER BY GRA_DATE DESC";
	my $result = $self->QueryAndFetchRows($query);
	if (!defined($result)) {
		$self->ErrorLog("CDBBenchmark::GetGraphList() failed");
		return 0;
	}

	return $result;
}

# ----------------------------
# Retrieve the specified Graph:
# ----------------------------
sub GraphGet
{
	my ($self,$grid) = @_;
	$grid = Util::SQLMakeIntSafe($grid);
	
	# graphs contains BLOB:
	if ($self->{'DBTYPE'} eq "Sybase") {
		$self->{'DBH'}->do('set textsize 100000');
	}
	else {
		$self->{'DBH'}->{'LongReadLen'} = 100000;
	}

	my $query = "SELECT * FROM T_BENCHMARK_GRAPH WHERE GRA_ID = $grid";
	my $result = $self->QueryAndFetchaRow($query);
	if (!defined($result)) {
		$self->ErrorLog("CDBBenchmark::GraphGet($grid) failed");
		return 0;
	}
	
	$result->{"GRA_NAME"} = Util::stripslashes($result->{"GRA_NAME"});
	$result->{"GRA_NOTE"} = Util::stripslashes($result->{"GRA_NOTE"});
	$result->{"GRA_BLOB"} = Util::stripslashes($result->{"GRA_BLOB"});
	
	if ($self->{'DBTYPE'} eq "Oracle" ||
	    $self->{'DBTYPE'} eq "Sybase") {
		$result->{"GRA_NAME"} =~ s/\'\'/\'/g;
		$result->{"GRA_NOTE"} =~ s/\'\'/\'/g;
		$result->{"GRA_BLOB"} =~ s/\'\'/\'/g;
	}
	return $result;
}

# --------------------------------
# Store a new graph:
# --------------------------------
sub StoreGraph
{
	my ($self,$name,$note,$graph,$w,$h) =  @_;

	$w     = Util::SQLMakeIntSafe($w);
	$h     = Util::SQLMakeIntSafe($h);

	$name  = Util::addslashes($name);
	$note  = Util::addslashes($note);
	$graph = Util::addslashes($graph);
	
	# graphs contains BLOB:
	if ($self->{'DBTYPE'} eq "Sybase") {
		$self->{'DBH'}->do('set textsize 100000');
	}
	else {
		$self->{'DBH'}->{'LongReadLen'} = 100000;
	}

	my $date = &Util::DateUnixToDB(time());
	my $query;

	if ($self->{'DBTYPE'} ne "Oracle") {
		if ($self->{'DBTYPE'} eq "Sybase") {
			$name  =~ s/\\\'/\'\'/g;
			$note  =~ s/\\\'/\'\'/g;
			$graph =~ s/\\\'/\'\'/g;
		}
		$query = "INSERT INTO T_BENCHMARK_GRAPH (GRA_NAME,GRA_DATE,GRA_NOTE,GRA_BLOB,GRA_WIDTH,GRA_HEIGHT) ".
					"VALUES ('$name','$date','$note','$graph',$w,$h)";
	}
	else {
		$name  =~ s/\\\'/\'\'/g;
		$note  =~ s/\\\'/\'\'/g;
		$graph =~ s/\\\'/\'\'/g;
		$query = "INSERT INTO T_BENCHMARK_GRAPH (GRA_ID,GRA_NAME,GRA_DATE,GRA_NOTE,GRA_BLOB,GRA_WIDTH,GRA_HEIGHT) ".
				"VALUES (GRA_ID_SEQ.NEXTVAL,'$name',TO_DATE('$date','YYYY/MM/DD HH24:MI:SS'),'$note','$graph',$w,$h)";
	}
				
	return $self->QueryExecute($query);
}

# --------------------------
# Delete the specified Graph:
# --------------------------
sub GraphDelete
{
	my ($self,$grid) = @_;

	$grid = Util::SQLMakeIntSafe($grid);
	
	my $query = "DELETE FROM T_BENCHMARK_GRAPH WHERE GRA_ID = $grid";
	return $self->QueryExecute($query);
}

#
# =================================================
# Function needed to compute statistics:
# =================================================
#
sub SetStartDate
{
	my ($self,$startdate) = @_;
	$self->{'m_startdate'} = $startdate;
}

sub SetEndDate
{
	my ($self,$enddate) = @_;
	$self->{'m_enddate'} = $enddate;
}

sub SetRequest
{
	my ($self,$req) = @_;
	$self->{'m_filterurl'} = $req;
}

sub SetServers
{
	my ($self,$server) = @_;
	$self->{'m_servers'} = $server;
	#if ($server ne "all") {
		#my @s = split(/./,$server);
		#$self->{'m_servers'} = ($s[0] << 24) + ($s[1] << 16) + ($s[2] << 8) + $s[3];
	#}
}

sub SetProtocols
{
	my ($self,$proto) = @_;
	$self->{'m_protocols'} = $proto;
}

sub SetThreshold
{
	my ($self,$t) = @_;
	$self->{'m_threshold'} = $t;
}

sub CheckDateRange
{
	my ($self,$year1,$month1,$day1,$year2,$month2,$day2) = @_;

	if ($year1 > $year2 ||
	    ($year1 == $year2 && $month1 > $month2) ||
	    ($year1 == $year2 && $month1 == $month2 && $day1 > $day2)) {
	    	return 0;
	}
	
	return 1;
}

sub NextDay
{
	my ($self,$curday) = @_;

	my ($year,$month,$day) = split(/[- ]/,$curday);
	my $nextday = mktime(0,0,0,$month,$day,$year) + 3600 * 24;
	return strftime("%Y-%m-%d",$nextday);
}

# -------------------------------------------------
# Helper functions used to build the requests:
# -------------------------------------------------
sub AddAvgFilter
{
	my ($self,$stat) = @_;
	my $filter = "";

	# always add the date constraint:
	$filter = " AND ".&{$m_db_features{$self->{'DBTYPE'}}{'DateCompare'}}("LOG_DATE",$self->{'m_startdate'},$self->{'m_enddate'});

	# and now the optional filters:
	if ($self->{'m_threshold'} != 0) {
		$filter .= " AND (BEN_SEC * 1000 + BEN_USEC / 1000) > ".$self->{'m_threshold'};
	}

	if ($self->{'m_filterurl'} ne "all" && $self->{'m_filterurl'} ne "") {
		$filter .= " AND URL_VALUE = \"".$self->{'m_filterurl'}."\"";
	}

	if ($self->{'m_servers'} ne "all" && $self->{'m_servers'} ne "") {
		$filter .= " AND BEN_SRVADDR = ".$self->{'m_servers'};
	}

	if ($stat ne "alert" && $self->{'m_protocols'} ne "all" && $self->{'m_protocols'} ne "") {
		$filter .= " AND BEN_SRVPORT = ".$self->{'m_protocols'};
	}

	return $filter;
}

sub AddMaxFilter
{	
	my ($self,$stat) = @_;
	return $self->AddAvgFilter($stat);
}

sub AddMinFilter
{	
	my ($self,$stat) = @_;
	return $self->AddAvgFilter($stat);
}

# =================================================
# GENERIC MAX STATISTICS
# =================================================
sub GetStat
{
	my ($self,$stat,$stattype,$scale) = @_;
	my $year;
	my $month;
	my $day;
	my $query;
	my $filter;
	my @result;

	# Get the query basis:
	if ($stattype eq "max") {
		$query = $m_maxQuery{$stat};
		$filter = $self->AddMaxFilter($stat);
	}
	elsif ($stattype eq "avg") {
		$query = $m_avgQuery{$stat};
		$filter = $self->AddAvgFilter($stat);
	}
	elsif ($stattype eq "min") {
		$query = $m_minQuery{$stat};
		$filter = $self->AddMinFilter($stat);
	}
	else {
		$self->ErrorLog("CDBBenchmark::GetStat() bad stattype: $stattype");
		return 0;
	}
		
	# The scale gives the loop count and the specific clause:
	my $startcount = 0;
	my $maxcount;
	if ($scale eq "minute" || $scale eq "hour") {
		$maxcount = 59;
	}
	elsif ($scale eq "day") {
		$maxcount = 23;
	}
	elsif ($scale eq "week") {
		$startcount = 1;
		$maxcount = 7;
	}
	elsif ($scale eq "month") {
		($year,$month,$day) = split(/[- ]/,$self->{'m_startdate'});
		$maxcount = $self->DayPerMonth($month,$year);
		$startcount = 1;
	}
	elsif ($scale eq "year") {
		$startcount = 1;
		$maxcount = 12;
	}
	else {
		$self->ErrorLog("CDBBenchmark::GetStat() bad scale: $scale");
		return 0;
	}
	
	# Loop on the interval:
	my $idx;
	my $q;
	my $res;

	for ( $idx = $startcount ; $idx <= $maxcount ; $idx++ ) {

		$q = $query;
	
		if ($scale eq "minute") {
			$q .= " WHERE ".&{$m_db_features{$self->{'DBTYPE'}}{'GetSecondPart'}}("LOG_DATE")." = $idx";
		}
		elsif ($scale eq "hour") {
			$q .= " WHERE ".&{$m_db_features{$self->{'DBTYPE'}}{'GetMinutePart'}}("LOG_DATE")." = $idx";
		}
		elsif ($scale eq "day") {
			$q .= " WHERE ".&{$m_db_features{$self->{'DBTYPE'}}{'GetHourPart'}}("LOG_DATE")." = $idx";
		}
		elsif ($scale eq "week") {
			$q .= " WHERE ".&{$m_db_features{$self->{'DBTYPE'}}{'GetDayOfWeek'}}("LOG_DATE")." = $idx";
		}
		elsif ($scale eq "month") {
			$q .= " WHERE ".&{$m_db_features{$self->{'DBTYPE'}}{'GetDayOfMonth'}}("LOG_DATE")." = $idx";
		}
		elsif ($scale eq "year") {
			$q .= " WHERE ".&{$m_db_features{$self->{'DBTYPE'}}{'GetMonth'}}("LOG_DATE")." = $idx";
		}

		$q .= $filter;

		# we don't use QueryAndFetchRows here... 
		$result[$idx - $startcount] = 0;
		my $sth = $self->{'DBH'}->prepare($q);
		if (!defined($sth)) { next; }

		$res = $sth->execute();
		if (!defined($res)) { next; }

		$res = $sth->fetchrow_arrayref();

		if (!defined($res)) {
			$self->ErrorLog("CDBBenchmark::GetStat($stat) failed");
			#return 0;
		}

		if (scalar(@{$res}) > 0 && defined($res->[0])) {
			$result[$idx-$startcount] = $res->[0];
		}

		$sth->finish();
	}

	return \@result;
}

# -------------------------------------------------
# Statistics per DAY OF MONTH:
# -------------------------------------------------
sub DayPerMonth
{
	my ($self,$month,$year) = @_;
	my $daycount = 31;

	if ($month == 4 || $month == 6 || $month == 9 || $month == 11) {
		$daycount = 30;
	}
	elsif ($month == 2) {
		# no matter with 2000 & 2100 !!!!
		if (($year % 4) == 0) { $daycount = 29; } else { $daycount = 28; }
	}

	return $daycount;
}

1;
# EOF
