#!/usr/bin/perl -w
use strict;
#
# ====================================================================
#
# Base class for SVG graph output
#
# Date: 14 Aug 2003
# Author: Jerome Delamarche (c) Copyright INODES.  2003  All rights reserved.
# Version: 2.0
#
# --------------------------------------------------------------------
#
# TODO:
#
# ====================================================================
#
use POSIX;

package CSVGGraph;

# ----------
# Ctor:
# ----------
sub new
{
	my $class = shift;
	my $self = {};
	bless($self);

	# the graph contains a plot area which has an offset from the origin:
	$self->{'graphicWidth'}          	= 0;
	$self->{'graphicHeight'}         	= 0;
	$self->{'plotWidth'}             	= 0;
	$self->{'plotHeight'}            	= 0;
	$self->{'plotOffsetX'}           	= 0;
	$self->{'plotOffsetY'}           	= 0;
	
	# different padding between borders and text:
	$self->{'outerPadding'}          	= 0;
	$self->{'outerPaddingX'}         	= 0;
	$self->{'outerPaddingY'}         	= 0;
	$self->{'innerPaddingX'}         	= 0;
	$self->{'innerPaddingY'}         	= 0;

	# X values:
	$self->{'dataX'}                 	= {};
	# Y values (2 dimension array)
	$self->{'dataY'}                 	= ();

	# textual values:
	$self->{'title'}                 	= '';
	$self->{'styleTitle'}            	= '';
	$self->{'styleTitleDefault'}     	= '';
	$self->{'labelX'}                	= '';
	$self->{'styleLabelX'}           	= '';
	$self->{'styleLabelXDefault'}    	= '';
	$self->{'labelY'}                	= '';
	$self->{'styleLabelY'}           	= '';
	$self->{'styleLabelYDefault'}    	= '';
	$self->{'tagsY'}			= ();

	$self->{'offsetGridlinesX'}      	= 0;
	$self->{'offsetGridlinesY'}      	= 0;

	# minY and maxY are used as "filters" to avoid "strange" values:
	$self->{'minY'}                  	= 0;
	$self->{'maxY'}                  	= 0;

	$self->{'resolutionY'}           	= 0;
	$self->{'decimalPlacesY'}        	= 0;
	$self->{'numGridlinesX'}         	= 0;
	$self->{'numGridlinesY'}         	= 0;
	$self->{'numTicksX'}		   	= 1;
	$self->{'numTicksY'}		   	= 1;
	$self->{'styleGridX'}            	= '';
	$self->{'styleGridXDefault'}     	= '';
	$self->{'styleGridY'}            	= '';
	$self->{'styleGridYDefault'}     	= '';

	# Presentation attributes:
	$self->{'styleBox'}              	= '';
	$self->{'styleBoxDefault'}       	= '';
	$self->{'styleTagsX'}            	= '';
	$self->{'styleTagsXDefault'}     	= '';
	$self->{'rotTagsX'}              	= 0;
	$self->{'styleTagsY'}            	= '';
	$self->{'styleTagsYDefault'}     	= '';
	$self->{'rotTagsY'}              	= 0;
	$self->{'styleLineDefault'}      	= '';
	$self->{'styleBarDefault'}       	= '';
	$self->{'stylePolylineDefault'}		= '';
	$self->{'extraSVG'}              	= '';
	
	$self->{'legendTitle'}		   	= '';
	$self->{'legendPosX'}			= 0;
	$self->{'legendPosY'}			= 0;
	$self->{'legendStyle'}		   	= '';
	$self->{'legendValue'}			= ();
	$self->{'legendValueStyle'}		= '';
	$self->{'legendText'}			= '';
	$self->{'legendFilter'}			= (); 

	# XML result !
	$self->{'svg'}                   	= '';

	# Last error message:
	$self->{'error'}                 	= '';

	$self->{'DECIMAL_POINT'}		= ".";
	$self->{'THOUSANDS_SEPARATOR'}		= "";

	return $self;
}


# ------------------
# Real init function:
# ------------------
sub init
{
	my ($self) = @_;
	my $graphEmpty = 0;
	my $text;
	
	$self->{'svgText'}   = ''; # SVG for outer text labels and title
	$self->{'svgPlot'}   = ''; # SVG for plot including tag text
	$self->{'numGridlinesX'} = $self->{'dataX'}{'xcount'};

	if ($self->{'numGridlinesX'} == 0) {
		$self->{'error'} = 'No data to plot.';
		return 0;
	}
	if ($self->{'numGridlinesY'} == 0) {
		$self->{'error'} = 'No range for Y values. Check number of Y ticks.';
		return 0;
	}
	
	my $width = $self->{'plotWidth'} - 1;
	my $height = $self->{'plotHeight'} - 1;

	# find range of all y values
	my ($min,$max) = $self->_findRange($self->{'minY'}, $self->{'maxY'}, $self->{'resolutionY'});
	$self->{'dataMaxY'} = $max;
	$self->{'dataMinY'} = $min;
	if (($self->{'dataMaxY'} - $self->{'dataMinY'}) == 0) {
		$graphEmpty = 1;
	}

	# find spacing for ticks and grid lines
	$self->{'deltaTicksX'} = $width / ($self->{'numGridlinesX'} - 1 + (2 * $self->{'offsetGridlinesX'}));
	$self->{'deltaTicksY'} = $height / ($self->{'numGridlinesY'} - 1 + (2 * $self->{'offsetGridlinesY'}));
	if ($graphEmpty == 1) {
		$self->{'factorY'} = 1;
	}
	else {
		$self->{'factorY'} = $height / ($self->{'dataMaxY'} - $self->{'dataMinY'});
	}

	# format text for tags on Y axis
	my $deltaTagsY = ($self->{'dataMaxY'} -  $self->{'dataMinY'}) / ($self->{'numGridlinesY'} - 1 + (2 * $self->{'offsetGridlinesY'}));
	if ($deltaTagsY == 0) { $deltaTagsY = 20; }	# empty graph ! 

	my $i;
	for ( $i = 0 ; $i < $self->{'numGridlinesY'} ; $i++ ) {
		$text = $self->{'dataMinY'} + $deltaTagsY * ($i +  $self->{'offsetGridlinesY'});
		# TODO: number_format !
		#$self->{'tagsY'}[$i] = number_format($text, $self->{'decimalPlacesY'},$self->{'DECIMAL_POINT'},$self->{'THOUSANDS_SEPARATOR'});
		$self->{'tagsY'}[$i] = $text;
	}
		
	@{$self->{'tagsY'}} = reverse @{$self->{'tagsY'}};
	if ($self->{'numGridlinesY'} == 0) {
		$self->{'error'} = 'No Y axis data.';
		return 0;
	}
		
	return 1;
}

# ------------------------------------------------------
# Draw the graph by computing the XML for each component:
# ------------------------------------------------------
sub drawGraph
{
	my ($self) = @_;
	$self->drawOuterText();
	$self->drawGridX();
	$self->drawGridY();
	$self->drawBox();
	$self->drawTagsX();
	$self->drawTagsY();
	$self->drawLegend();
}

# ------------------------------
# Draw the title and axis labels:
# ------------------------------
sub drawOuterText
{
	my ($self) = @_;
	my $offset;
	my $transform;

	# draw title
	if ($self->{'title'} ne "") { # check if there is something to draw
		$offset         = $self->{'plotOffsetX'} + $self->{'plotWidth'} / 2;
		$transform      = "transform='translate(0, $self->{'outerPadding'})'";
		$self->{'svgText'} .= "<text text-anchor='middle' $transform x='$offset' y='1em' ";
		$self->{'svgText'} .= "style='".$self->{'styleTitleDefault'}.$self->{'styleTitle'}."'>";
		$self->{'svgText'} .= $self->{'title'}."</text>\n";
	}

	# draw label Y
	if ($self->{'labelY'} ne "") { # check if there is something to draw
		$offset         = $self->{'plotOffsetY'} + ($self->{'plotHeight'} / 2);
		$transform      = "transform='translate($self->{'outerPaddingY'}, 0) rotate(-90 0 $offset)'";
		$self->{'svgText'} .= "<text text-anchor='middle' $transform x='0' y='$offset' dy='1em' ";
		$self->{'svgText'} .= "style='".$self->{'styleLabelYDefault'}.$self->{'styleLabelY'}."'>";
		$self->{'svgText'} .= $self->{'labelY'}."</text>\n";
	}

	# draw label X
	if ($self->{'labelX'} ne "") { # check if there is something to draw
		$offset         = $self->{'plotOffsetX'} + ($self->{'plotWidth'} / 2);
		$transform      = "transform='translate(0, -$self->{'outerPaddingX'})'";
		$self->{'svgText'} .= "<text text-anchor='middle' $transform x='$offset' y='".$self->{'graphicHeight'}."' ";
		$self->{'svgText'} .= "style='".$self->{'styleLabelXDefault'}.$self->{'styleLabelX'}."'>";
		$self->{'svgText'} .= $self->{'labelX'}."</text>\n";
	}
}

# ---------------------
# Draw the X grid lines:
# ---------------------
sub drawGridX
{
	my ($self) = @_;
	my $u;
	my $idx;

	$self->{'svgPlot'} .= "<g style='".$self->{'styleGridXDefault'}.$self->{'styleGridX'}."'>\n";
	my $top = 0;
	my $bottom = $self->{'plotHeight'} - 1;

	for ( $idx = 0 ; $idx < $self->{'dataX'}{'xcount'} ; $idx++ ) {
		if (defined($self->{'dataX'}{$idx}) && $self->{'dataX'}{$idx} ne "") {
			$u = $self->{'deltaTicksX'} * ($idx +  $self->{'offsetGridlinesX'});
			$self->{'svgPlot'} .= "<line x1='$u' y1='$top' x2='$u' y2='$bottom'/>\n";
		}
	}
	
	$self->{'svgPlot'} .= "</g>\n";
}

# ---------------------
# Draw the Y grid lines:
# ---------------------
sub drawGridY
{
	my ($self) = @_;
	my $idx;
	my $v;

	$self->{'svgPlot'} .= "<g style='".$self->{'styleGridYDefault'}.$self->{'styleGridX'}."'>\n";
	my $left = 0;
	my $right = $self->{'plotWidth'} - 1;

	for ( $idx = 0 ; $idx < $self->{'numGridlinesY'} ; $idx++ ) {
		$v = $self->{'deltaTicksY'} * ($idx +  $self->{'offsetGridlinesY'});
		$self->{'svgPlot'} .= "<line x1='$left' y1='$v' x2='$right' y2='$v'/>\n";
	}

	$self->{'svgPlot'} .= "</g>\n";
}

# -------------------------------------
# Draw the box around the plotting area:
# -------------------------------------
sub drawBox
{
	my ($self) = @_;

	my $width = $self->{'plotWidth'} - 1;
	my $height = $self->{'plotHeight'} - 1;

	$self->{'svgPlot'} .= "<g style='".$self->{'styleBoxDefault'}.$self->{'styleBox'}."'>\n";
	$self->{'svgPlot'} .= "<polyline points='0,0 $width,0 $width,$height 0,$height 0,0'></polyline>\n";
	$self->{'svgPlot'} .= "</g>\n";
}

# ---------------------------------------------
# Draw the X axis tag outside the plotting area:
# --------------------------------------------
sub drawTagsX
{
	my ($self) = @_;

	$self->{'svgPlot'} .= "<g style='".$self->{'styleTagsXDefault'}.$self->{'styleTagsX'}."'>\n";
	my $bottom = $self->{'plotHeight'} - 1;
	my $innerPadding = $self->{'innerPaddingX'};
	my $idx;
	my $u;
	my $transform;
	my $text;

	for ( $idx = 0 ; $idx < $self->{'dataX'}{'xcount'} ; $idx++ ) {
		if (!defined($self->{'dataX'}{$idx})) { next; }

		$text = $self->{'dataX'}{$idx};
			
		$u = $self->{'deltaTicksX'} * ($idx +  $self->{'offsetGridlinesX'});

		if ($self->{'rotTagsX'} == 0) {
			$transform = "transform='translate(0, $innerPadding)'";
			$self->{'svgPlot'} .= "<text text-anchor='middle' $transform dy='1em' x='$u' y='$bottom'>$text</text>\n";
		} elsif($self->{'rotTagsX'} > 0) {
			$transform = "transform='translate(0, $innerPadding) rotate(".$self->{'rotTagsX'}." $u $bottom)'";
			$self->{'svgPlot'} .= "<text text-anchor='start' $transform x='$u' y='$bottom'>$text</text>\n";
		} else {
			$transform = "transform='translate(0, $innerPadding) rotate(".$self->{'rotTagsX'}." $u $bottom)'";
			$self->{'svgPlot'} .= "<text text-anchor='end' $transform x='$u' y='$bottom'>$text</text>\n";
		}
	}

	$self->{'svgPlot'} .= "</g>\n";
}

# ---------------------------------------------
# Draw the Y axis tag outside the plotting area:
# --------------------------------------------
sub drawTagsY
{
	my ($self) = @_;
	my $i;
	my $v;
	my $text;
	my $transform;

	if ((scalar @{$self->{'tagsY'}}) == 0) {
		return; # no data to plot. error should be picked up by init method.
	}

	$self->{'svgPlot'} .= "<g style='".$self->{'styleTagsYDefault'}.$self->{'styleTagsY'}."'>\n";
	my $innerPadding = $self->{'innerPaddingY'};

	for ( $i = 0 ; $i < $self->{'numGridlinesY'} ; $i++ ) {
		$text = $self->{'tagsY'}[$i];
		$v = $self->{'deltaTicksY'} * ($i +  $self->{'offsetGridlinesY'});
		if ($self->{'rotTagsY'} == 0) {
			$transform = "transform='translate(-$innerPadding, 0)'";
			$self->{'svgPlot'} .= "<text text-anchor='end' $transform x='0' dy='0.5em' y='$v'>$text</text>\n";
		} else {
			$transform = "transform='translate(-$innerPadding, 0) rotate(".$self->{'rotTagsY'}." 0 $v)'";
			$self->{'svgPlot'} .= "<text text-anchor='end' $transform x='0' dy='0.5em' y='$v'>$text</text>\n";
		}
	}

	$self->{'svgPlot'} .= "</g>\n";
}

# ---------------
# Draw the legend:
# ---------------
sub drawLegend
{	
	my ($self) = @_;
	my $x2;
	my $ypos;
	my $m;

	# compute the legend vertical position:
	my $markerHeight = 44;
	my $lposX = 0;
	my $lposY = ($self->{'graphicHeight'} - (scalar(@{$self->{'legendValue'}}) * 13 + 40 + $markerHeight)) / 2;
	my $lwidth = 76;
	my $lheight = $lposY + scalar(@{$self->{'legendValue'}}+1) * 13 + $markerHeight;
	
	# display the title
	$self->{'legendText'} .= "<text style='".$self->{'legendStyle'}."' x='$lposX' y='$lposY'>".$self->{'legendTitle'}."</text>\n";
	$self->{'legendText'} .= "<g style='".$self->{'legendValueStyle'}."'>\n";
	
	# surround the legend with a box:
	$lposY += 8;
	$self->{'legendText'} .= "<polyline style='".$self->{'styleBoxDefault'}.$self->{'styleBox'}." filter:url(#dropShadow);' ".
					"points='$lposX,$lposY $lposX,$lheight, $lwidth,$lheight $lwidth,$lposY $lposX,$lposY'></polyline>";

	# display the legend info:
	my $style;
	my $text;
	$lposX += 3;
	for ( $m = 0 ; $m < scalar(@{$self->{'legendValue'}}) ; $m++ ) {
		$style = $self->{'legendValue'}[$m]{'style'};
		$text  = $self->{'legendValue'}[$m]{'text'};
		$x2 = $lposX + 10;
		$ypos = $lposY + 12 + $m * 13;
		$self->{'legendText'} .= "<line style='$style' x1='$lposX' y1='$ypos' x2='$x2' y2='$ypos'></line>";
		$ypos += 3;
		$x2 += 2;
		$self->{'legendText'} .= "<text x='$x2' y='$ypos'>$text</text>\n";
	}
	
	# Display the marker info:
	$ypos += 7;
	$x2 = $lwidth - 6;
	$self->{'legendText'} .= "<line x1='$lposX' y1='$ypos' x2='$x2' y2='$ypos'></line>";
	$lposY += 20 + @{$self->{'legendValue'}} * 13;
	my @stroketype = ( "", "stroke-dasharray: 2,2;", "stroke-dasharray: 6,2;");
	my @markertext = ( "Average", "Min", "Max" );
	for ( $m = 0 ; $m < 3 ; $m++ ) {
		$ypos = $lposY + $m * 10;
		$x2 = $lposX + 10;
		$self->{'legendText'} .= "<line style='stroke: black; ".$stroketype[$m]."stroke-width:2; shape-rendering: geometricPrecision; fill:none' x1='$lposX' y1='$ypos' x2='$x2' y2='$ypos'></line>";
		$ypos += 3;
		$x2 += 2;
		$self->{'legendText'} .= "<text x='$x2' y='$ypos'>".$markertext[$m]."</text>\n";
	}
	
	# Display the filters:
	my $yposval = $lheight + 14;
	my $xposval = 35;

	for ( $m = 0 ; $m < scalar(@{$self->{'legendFilter'}}) ; $m++ ) {
		$self->{'legendText'} .= "<text x='0' y='$yposval'>".$self->{'legendFilter'}[$m]{'Filtername'}.":</text>\n";
		$self->{'legendText'} .= "<text x='$xposval' y='$yposval'>".$self->{'legendFilter'}[$m]{'Value'}."</text>\n";
		$yposval += 12;
	}
	
	$self->{'legendText'} .= "</g>\n";
}

# ------------------------------------------
# Draw a line from point to point.
# Draw the shape at each intersection.
# ------------------------------------------
sub polyline
{
	my ($self,$whichDataSet) = @_;
	my $attributes;

	if (!defined($self->{'format'}[$whichDataSet]{'style'})) {
		$self->{'error'} = 'No style defined for data plot.';
		return 0; 
	}

	if (!defined($self->{'format'}[$whichDataSet]{'attributes'})) { 
		$attributes = "";
	}
	else {
		$attributes = $self->{'format'}[$whichDataSet]{'attributes'};
	}

	$self->{'svgPlot'} .= "<g style='".$self->{'stylePolylineDefault'}.$self->{'format'}[$whichDataSet]{'style'}."'>\n";
	$self->{'svgPlot'} .= "<polyline $attributes ";

	my $i;
	my $u;
	my $v;
	my $x;
	my $y;
	my $oldU;
	my $oldV;

	for ( $i = 0 ; $i < $self->{'dataX'}{'xcount'} ; $i++ ) {
		$x = $self->{'dataX'}{$i};
		$y = $self->{'dataY'}[$whichDataSet]{$i};

		$u = $self->{'deltaTicksX'} * ($i +  $self->{'offsetGridlinesX'});
		$v = $self->{'factorY'} * ($y - $self->{'dataMinY'});
		$v = $self->{'plotHeight'} - $v;

		if ($i == 0) {
			$self->{'svgPlot'} .= "points='$u,$v";
		}
		else  {
			$self->{'svgPlot'} .= " $u,$v ";
		}

		$oldU = $u;
		$oldV = $v;
	}

	$self->{'svgPlot'} .= "'/>\n</g>\n";

	return 1;
}

# ------------------------------------------
# Draw a line from point to point.
# Draw the shape at each intersection.
# ------------------------------------------
sub line
{
	my ($self,$whichDataSet) = @_;
	my $attributes;

	if (!defined($self->{'format'}[$whichDataSet]{'style'})) {
		$self->{'error'} = "No style defined for data plot ($whichDataSet). Check parameters.";
		return 0; 
	}


	if ($self->{'format'}[$whichDataSet]{'attributes'} eq "") { 
		$attributes = "";
	}
	else {
		$attributes = $self->{'format'}[$whichDataSet]{'attributes'};
	}

	$self->{'svgPlot'} .= "<g style='".$self->{'styleLineDefault'}.$self->{'format'}[$whichDataSet]{'style'}."'>\n";

	my $i;
	my $u;
	my $v;
	my $x;
	my $y;
	my $oldU;
	my $oldV;

	for ( $i = 0 ; $i < $self->{'dataX'}{'xcount'} ; $i++ ) {
		$x = $self->{'dataX'}{$i};
		$y = $self->{'dataY'}[$whichDataSet]{$i};

		$u = $self->{'deltaTicksX'} * ($i +  $self->{'offsetGridlinesX'});
		$v = $self->{'factorY'} * ($y - $self->{'dataMinY'});
		$v = $self->{'plotHeight'} - $v;
		
		if ($i == 0) {
			$oldU = $u;
			$oldV = $v;
		}
		$self->{'svgPlot'} .= "<line $attributes x1='$oldU' y1='$oldV' x2='$u' y2='$v'/>\n";
		$oldU = $u;
		$oldV = $v;
	}

	$self->{'svgPlot'} .= "</g>\n";

	return 1;
}

# -----------------------------------
# Draw a bar for each data of the set:
# -----------------------------------
sub bar
{
	my ($self,$whichDataSet) = @_;

	if (!defined($self->{'format'}[$whichDataSet]{'style'})    ||
		!isset($self->{'format'}[$whichDataSet]{'barWidth'}) ||
		!isset($self->{'format'}[$whichDataSet]{'barOffset'}  )) {
		$self->{'error'} = 'Style parameters missing for bar plot.';
		return 0; 
	}

	$self->{'svgPlot'} .= "<g style='".$self->{'styleBarDefault'}.$self->{'format'}[$whichDataSet]{'style'}."'>\n";
	my $barWidth = $self->{'format'}[$whichDataSet]{'barWidth'} * $self->{'deltaTicksX'};
	my $barOffset = $self->{'format'}[$whichDataSet]{'barOffset'} * $barWidth;

	my $i;
	my $x;
	my $y;
	my $u;
	my $v;
	my $height;

	for ( $i = 0 ; $i < $self->{'dataX'}{'xcount'} ; $i++ ) {
		$x = $self->{'dataX'}{$i};
		$y = $self->{'dataY'}[$whichDataSet]{$i};

		$u = $self->{'deltaTicksX'} * ($i + $self->{'offsetGridlinesX'}) - ($barWidth / 2) + $barOffset;
		$v = $self->{'factorY'} * ($y - $self->{'dataMinY'});
		$v = $self->{'plotHeight'} - $v;
		$height = $self->{'factorY'} * ($y - $self->{'dataMinY'}) - 1;

		$self->{'svgPlot'} .= "<rect x='$u' y='$v' width='$barWidth' height='$height'/>\n";
	}

	$self->{'svgPlot'} .= "</g>\n";

	return 1;
}

# ---------------------------------------------------------------------------
# Determine the range of values for a dataset
# ---------------------------------------------------------------------------
sub _findRange
{
	my ($self,$minY,$maxY,$resolution) = @_;

	my $min = $minY;
	my $max = $maxY;
	my $i;
	my $j;
	my $factor;
	my $ycount = scalar @{$self->{'dataY'}};

	if ($ycount == 0) {  return {'min' => 0, 'max' => 0}; }

	for ( $i = 1 ; $i < $ycount ; $i++ ) {
		my $ref = $self->{'dataY'}[$i];
		my $ycount2 = $ref->{'xcount'};

		for ( $j = 0 ; $j < $ycount2 ; $j++ ) {
			my $value = $ref->{$j};
			if ($value eq "none") { next; }
			if ($value > $max) { $max = $value; }
			if ($value < $min) { $min = $value; }
		}
	}

	if ($max == 0) {
		$factor = 1;
	} else {
		if ($max < 0) { $factor = - POSIX::pow(10, (POSIX::floor(POSIX::log10(POSIX::abs($max))) + $resolution) ); }
		else { $factor = POSIX::pow(10, (POSIX::floor(POSIX::log10(POSIX::abs($max))) - $resolution) ); }
	}
	$max = $factor * POSIX::ceil($max / $factor);
	$min = $factor * POSIX::floor($min / $factor);

	return ($min,$max);
}

# -----------------------------------------
# OK: Generate the XML for the entire graph:
# -----------------------------------------
sub generateSVG
{
	my ($self) = @_;

	$self->{'svg'}  = "<?xml version='1.0' encoding='iso-8859-1'?>\n";
	$self->{'svg'} .= "<!DOCTYPE svg PUBLIC '-//W3C//DTD SVG 20000303 Stylable//EN' ";
	$self->{'svg'} .= "'http://www.w3.org/TR/2000/03/WD-SVG-20000303/DTD/svg-20000303-stylable.dtd'>\n";
	$self->{'svg'} .= "<svg width='".$self->{'graphicWidth'}."' height='".$self->{'graphicHeight'}."'>\n";

	if ($self->{'extraSVG'} ne "") {
		$self->{'svg'} .= "<!-- Extra SVG -->\n";
		$self->{'svg'} .= $self->{'extraSVG'}."\n";
		$self->{'svg'} .= "<!-- End Extra SVG -->\n";
	}

	if ($self->{'svgText'} ne "") {
		$self->{'svg'} .= "<!-- Outer Text -->\n";
		$self->{'svg'} .= "<g>\n";
		$self->{'svg'} .= $self->{'svgText'}."\n";
		$self->{'svg'} .= "</g>\n" ;
		$self->{'svg'} .= "<!-- End Outer Text -->\n";
	}
		
	if ($self->{'legendText'} ne "") {
		$self->{'svg'} .= "<!-- Start Legend -->\n";
		$self->{'svg'} .= $self->{'legendText'}."\n";
		$self->{'svg'} .= "<!-- End Legend -->\n";
	}

	$self->{'svg'} .= "<!-- Plot Area -->\n";
	$self->{'svg'} .= "<g transform='translate(".$self->{'plotOffsetX'}.",".$self->{'plotOffsetY'}.")'>\n";
	$self->{'svg'} .= $self->{'svgPlot'}."\n";
	$self->{'svg'} .= "</g>\n" ;
	$self->{'svg'} .= "<!-- End Plot Area -->\n";

	$self->{'svg'} .= "</svg>\n" ;
}

1;

# EOF
